<?php

namespace Drupal\niobi_app;

use Drupal\Core\Session\AccountInterface;
use Drupal\niobi_app\Entity\NiobiApplicationInterface;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflow;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflowStage;
use Drupal\niobi_app\Plugin\task\Bundle\ApplicationReviewTask;
use Drupal\niobi_app\Entity\NiobiApplication;
use Drupal\task\Entity\Task;
use Drupal\user\UserInterface;

class NiobiAppUtilities {

  public static function getDecisionsAsOptions() {
    $plugin_manager = \Drupal::service('plugin.manager.niobi_app_decision');
    $plugin_definitions = $plugin_manager->getDefinitions();
    $ret = [];
    foreach ($plugin_definitions as $pd) {
      if(isset($pd['id'])) {
        $ret[$pd['id']] = $pd['label'];
      }
    }
    return $ret;
  }

  /**
   * @param UserInterface $reviewer
   * @param NiobiApplication $niobi_application
   * @return bool
   *   FALSE means they are not excluded, and OK to assign.
   *   TRUE means they are excluded, and should not be assigned.
   */
  public static function ExcludedAsReviewer(UserInterface $reviewer, NiobiApplication $niobi_application) {
    $workflow_stage = $niobi_application->getCurrentApplicationStage();
    $plugin_manager = \Drupal::service('plugin.manager.niobi_app');
    $plugin_definitions = $plugin_manager->getDefinitions();

    // If the user meets an exclusion rule, return that they are excluded.
    foreach ($plugin_definitions as $plugin_definition) {
      if ($plugin_definition['class']::isExcluded($reviewer, $niobi_application, $workflow_stage)) {
        return TRUE;
      }
    }
    // Reviewer was not excluded by any plugin.
    return FALSE;
  }

  /**
   * @param $status
   * @param NiobiApplicationWorkflowStage|NULL $workflow_stage
   * @return mixed
   */
  public static function getStageName($status, NiobiApplicationWorkflowStage $workflow_stage = NULL) {
    $stages = NiobiAppUtilities::generateStageListForWorkflow($workflow_stage);

    return $stages[$status];
  }

  /**
   * @param NiobiApplicationWorkflowStage|NULL $workflow_stage
   * @return array
   */
  public static function generateStageListForWorkflow(NiobiApplicationWorkflowStage $workflow_stage = NULL) {
    $stages = [
      'nomination' => t('Nomination'),
      'nomination_claim' => t('Claim Nomination'),
      'application' => t('Application'),
      'review' => t('In Review'),
      'decision' => t('Decision Pending'),
      'complete' => t('Complete'),
    ];

    if ($workflow_stage) {
      if (empty($workflow_stage->getNominationForm())) {
        unset($stages['nomination']);
        unset($stages['nomination_claim']);
      }
      if (empty($workflow_stage->getApplicationForm())) {
        unset($stages['application']);
      }
      foreach ($stages as $key => $label) {
        $field = 'field_label_for_' . $key;
        $val = $workflow_stage->get($field)->getValue();
        if (isset($val[0]['value']) && !empty($val[0]['value'])) {
          $stages[$key] = $val[0]['value'];
        }
      }
    }

    return $stages;
  }

  /**
   * @param NiobiApplicationWorkflowStage|NULL $workflow_stage
   * @return array
   */
  public static function generateStageListForChecklists(NiobiApplicationWorkflowStage $workflow_stage = NULL) {
    $stages = NiobiAppUtilities::generateStageListForWorkflow($workflow_stage);
    if (!empty($workflow_stage->getRequiredAddendaForms())) {
      $stages['required_addenda'] = t('Required Addenda');
    }
    if (!empty($workflow_stage->getOptionalAddendaForms())) {
      $stages['optional_addenda'] = t('Optional Addenda');
    }
    $support = $workflow_stage->getSupportLetterInformation();
    $support_name = t('Support Letters');
    if (!empty($support['name'])) {
      $support_name = $support['name'];
    }
    $labels['support'] = $support_name;
    $labels['support_request'] = t('Request for %letter', ['%letter' => $support_name]);
    return $stages;
  }

  /**
   * @param $status
   * @param NiobiApplicationInterface $application
   * @return string
   */
  public static function generateStageList($status, NiobiApplicationInterface $application) {
    $ret = '<div class="btn-group">';
    $stages = NiobiAppUtilities::generateStageListForWorkflow($application->getCurrentApplicationStage());
    $stage_identified = FALSE;

    foreach ($stages as $stage => $label) {
      if ($stage === $status) {
        $ret .= '<div class="btn btn-primary">' . $label . '</div>';
        $stage_identified = TRUE;
      }
      else {
        if ($stage_identified) {
          $ret .= '<div class="btn btn-default">' . $label . '</div>';
        }
        else {
          $ret .= '<div class="btn btn-info">' . $label . '</div>';
        }
      }
    }
    $ret .= '</div>';

    return $ret;
  }

  /**
   * @param NiobiApplication $application
   * @param NiobiApplicationWorkflow $workflow
   * @return array
   */
  public static function generateReviewAssignmentRenderArray (NiobiApplication $application, NiobiApplicationWorkflow $workflow) {
    if ($workflow->isReadyToUse()) {
      $status = $application->get('field_application_status')->getValue()[0]['value'];
      $status_name = t('Current Status') . ': ' . NiobiAppUtilities::getStageName($status);
      $output = [
        'header' => [
          '#type' => 'html_tag',
          '#tag' => 'p',
          'value' => [
            '#type' => 'html_tag',
            '#tag' => 'strong',
            '#value' => t($status_name),
          ],
        ],
      ];
      $query = \Drupal::entityQuery('task');
      $query->condition('type', 'application_review_task');
      $query->condition('field_application', $application->id());
      $result = $query->execute();
      if(count($result)) {
        foreach ($result as $assignment) {
          $task = Task::load($assignment);
          if ($task) {
            $task_status = ApplicationReviewTask::getReviewTaskStatus($task, $application);
            $add = [
              '#type' => 'html_tag',
              '#tag' => 'div',
              'value' => [
                '#type' => 'html_tag',
                '#tag' => 'span',
                '#value' => $task->label(),
              ],
            ];
            switch ($task_status) {
              case 'complete':
                $add['value']['#attributes'] = ['class' => 'btn btn-xs btn-success'];
                $output[$task->id()] = $add;
                break;
              case 'draft':
                $add['value']['#attributes'] = ['class' => 'btn btn-xs btn-warning'];
                $output[$task->id()] = $add;
                break;
              case 'not started':
                $add['value']['#attributes'] = ['class' => 'btn btn-xs btn-danger'];
                $output[$task->id()] = $add;
                break;
            }
          }
        }
      }
    }
    else {
      \Drupal::messenger()->addError('The application has not been fully set up. Please contact the form administrator.');
      $output = '<p><em>' . t('The application system is not fully set up.') . '</em></p>';
      return ['#type' => 'markup', '#markup' => $output];
    }

    return [$output];
  }

  public static function isWorkflowAdmin($workflow_id, AccountInterface $account = NULL) {
    $account = $account ? $account : \Drupal::currentUser();
    $workflow = NiobiApplicationWorkflow::load($workflow_id);
    $admin_team = $workflow->getAdminTeam();
    $is_admin = in_array($account, $admin_team);
    $permission = $account->hasPermission('administer niobi_app applications');
    return $is_admin || $permission;
  }

}